colors: dict[str, str] = {
    'Letusan/Erupsi': '#F44336',
    'Awan Panas Letusan': '#e91e63',
    'Awan Panas Guguran': '#673ab7',
    'Guguran': '#1976d2',
    'Hembusan': '#3f51b5',
    'Harmonik': '#607d8b',
    'Tremor Non-Harmonik': '#0d47a1',
    'Tornillo': '#03a9f4',
    'Low Frequency': '#006064',
    'Hybrid/Fase Banyak': '#009688',
    'Vulkanik Dangkal': '#8BC34A',
    'Vulkanik Dalam': '#33691E',
    'Very Long Period': '#827717',
    'Tektonik Lokal': '#F57F17',
    'Terasa': '#FFCA28',
    'Tektonik Jauh': '#FFA726',
    'Double Event': '#ff5722',
    'Getaran Banjir': '#795548',
    'Deep Tremor': '#333333',
    'Tremor Menerus': '#9E9E9E',
}

columns: dict[str, str] = {
    'gempa.letusan_erupsi': 'Letusan/Erupsi',
    'gempa.awan_panas_letusan': 'Awan Panas Letusan',
    'gempa.awan_panas_guguran': 'Awan Panas Guguran',
    'gempa.guguran': 'Guguran',
    'gempa.hembusan': 'Hembusan',
    'gempa.harmonik': 'Harmonik',
    'gempa.tremor_non-_harmonik': 'Tremor Non-Harmonik',
    'gempa.tornillo': 'Tornillo',
    'gempa.low_frequency': 'Low Frequency',
    'gempa.hybrid_fase_banyak': 'Hybrid/Fase Banyak',
    'gempa.vulkanik_dangkal': 'Vulkanik Dangkal',
    'gempa.vulkanik_dalam': 'Vulkanik Dalam',
    'gempa.very_long_period': 'Very Long Period',
    'gempa.tektonik_lokal': 'Tektonik Lokal',
    'gempa.terasa': 'Terasa',
    'gempa.tektonik_jauh': 'Tektonik Jauh',
    'gempa.double_event': 'Double Event',
    'gempa.getaran_banjir': 'Getaran Banjir',
    'gempa.deep_tremor': 'Deep Tremor',
    'gempa.tremor_menerus': 'Tremor Menerus'
}

event_codes = ['*', 'lts', 'apl', 'apg', 'gug', 'hbs', 'hrm', 'tre',
               'tor', 'lof', 'hyb', 'vtb', 'vta', 'vlp', 'tel', 'trs',
               'tej', 'dev', 'gtb', 'dpt', 'mtr']

event_names: dict[str, str] = {
    'lts': 'Letusan/Erupsi',
    'apl': 'Awan Panas Letusan',
    'apg': 'Awan Panas Guguran',
    'gug': 'Guguran',
    'hbs': 'Hembusan',
    'hrm': 'Harmonik',
    'tre': 'Tremor Non-Harmonik',
    'tor': 'Tornillo',
    'lof': 'Low Frequency',
    'hyb': 'Hybrid/Fase Banyak',
    'vtb': 'Vulkanik Dangkal',
    'vta': 'Vulkanik Dalam',
    'vlp': 'Very Long Period',
    'tel': 'Tektonik Lokal',
    'trs': 'Terasa',
    'tej': 'Tektonik Jauh',
    'dev': 'Double Event',
    'gtb': 'Getaran Banjir',
    'dpt': 'Deep Tremor',
    'mtr': 'Tremor Menerus'
}
